%module libgraph
%{
#include <libgraph.h>

typedef graphStruct Graph;
typedef graphPieStruct Pie;
typedef graphBarStruct Bar;
typedef graphLineStruct Line;
typedef graphMultiBarStruct MultiBar;
typedef graphMultiLineStruct MultiLine;
%}

// libgraphcolor.h
typedef enum {
  DEFAULT = -1,
  BLACK = 0,
  WHITE,
  GREY,
  DKGREY,
  GREEN,
  ORANGE,
  BLUE,
  RED,
  CYAN,
  YELLOW
} graphColor;

%name(color_init) void lg_graph_color_init (gdImagePtr im, int **colarrayptr);
%name(color_next) int lg_graph_color_next (graphColor curr);


// libgraphbase.h
typedef struct lg_graph {
  %addmethods {
	Graph(char *title, int width, int height) {
	  lg_graph_new(title, width, height);
	}
	~Graph() {
	  lg_graph_destroy (self);
	}
	char* get_title();
	void set_title(const char *title);
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size(self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y);
	int get_bg_color();
	void set_bg_color(int color);
	graphType get_type();
	void set_type(graphType type);
	int add_pointname (char *name);
	void render ();
	void save (char *name);
  }
} Graph;

// libgraphpie.h
typedef struct lg_graph_pie {
  %addmethods {
	Pie(char *title, int width, int height) {
	  lg_graph_pie_new(title, width, height);
	}
	~Pie() {
	  lg_graph_pie_destroy (self);
	}
	int add_slice (char *name , int value);
	int add_slice_dbl (char *name , double value);
	void render ();

	// Graph
	char* get_title() {
	  lg_graph_get_title((Graph *)self);
	}
	void set_title(const char *title) {
	  lg_graph_set_title((Graph *)self, title);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size((Graph *)self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y) {
	  lg_graph_set_size((Graph *)self, x, y);
	}
	int get_bg_color() {
	  lg_graph_get_bg_color((Graph *)self);
	}
	void set_bg_color(int color) {
	  lg_graph_set_bg_color((Graph *)self, color);
	}

	int add_pointname (char *name) {
	  lg_graph_add_pointname((Graph *)self, name);
	}
	void save (char *name) {
	  lg_graph_save((Graph *)self, name);
	}
  }
} Pie;

// libgraphbar.h
typedef struct lg_graph_bar {
  %addmethods {
	Bar(char *title, int width, int height) {
	  lg_graph_bar_new(title, width, height);
	}
	~Bar() {
	  lg_graph_bar_destroy (self);
	}
	int add_col (char *name, int value);
	int add_col_dbl (char *name, double value);
	void render ();

	// Graph
	char* get_title() {
	  lg_graph_get_title((Graph *)self);
	}
	void set_title(const char *title) {
	  lg_graph_set_title((Graph *)self, title);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size((Graph *)self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y) {
	  lg_graph_set_size((Graph *)self, x, y);
	}
	int get_bg_color() {
	  lg_graph_get_bg_color((Graph *)self);
	}
	void set_bg_color(int color) {
	  lg_graph_set_bg_color((Graph *)self, color);
	}

	int add_pointname (char *name) {
	  lg_graph_add_pointname((Graph *)self, name);
	}
	void save (char *name) {
	  lg_graph_save((Graph *)self, name);
	}
  }
} Bar;

// libgraphline.h
typedef struct lg_graph_line {
  %addmethods {
	Line(char *title, int width, int height) {
	  lg_graph_line_new(title, width, height);
	}
	~Line() {
	  lg_graph_line_destroy (self);
	}
	int add_point (char *name, int value);
	int add_point_dbl (char *name, double value);
	void render ();

	// Graph
	char* get_title() {
	  lg_graph_get_title((Graph *)self);
	}
	void set_title(const char *title) {
	  lg_graph_set_title((Graph *)self, title);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size((Graph *)self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y) {
	  lg_graph_set_size((Graph *)self, x, y);
	}
	int get_bg_color() {
	  lg_graph_get_bg_color((Graph *)self);
	}
	void set_bg_color(int color) {
	  lg_graph_set_bg_color((Graph *)self, color);
	}

	int add_pointname (char *name) {
	  lg_graph_add_pointname((Graph *)self, name);
	}
	void save (char *name) {
	  lg_graph_save((Graph *)self, name);
	}
  }
} Line;

// libgraphline.h
typedef struct lg_graph_multibar {
  %addmethods {
	MultiBar(char *title, int width, int height, int groupsize) {
	  lg_graph_multibar_new(title, width, height, groupsize);
	}
	~MultiBar() {
	  lg_graph_multibar_destroy (self);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(add_col) VALUE call_func_add_col(VALUE name, VALUE value) {
	  int i, len;
	  int *pnt;

	  Check_Type(value, T_ARRAY);
	  len = RARRAY(value)->len;
	  pnt = ALLOCA_N(int, len);
	  for (i = 0; i < len; i++) {
		pnt[i] = NUM2INT(RARRAY(value)->ptr[i]);
	  }
	  return INT2FIX(lg_graph_multibar_add_col (self, STR2CSTR(name), pnt));
	}
	%name(add_col_dbl) VALUE call_func_add_col_dbl(VALUE name, VALUE value) {
	  int i, len;
	  double *pnt;

	  Check_Type(value, T_ARRAY);
	  len = RARRAY(value)->len;
	  pnt = ALLOCA_N(double, len);
	  for (i = 0; i < len; i++) {
		pnt[i] = NUM2DBL(RARRAY(value)->ptr[i]);
	  }
	  return INT2FIX(lg_graph_multibar_add_col_dbl (self, STR2CSTR(name), pnt));
	}
%endif
	void render ();

	// Graph
	char* get_title() {
	  lg_graph_get_title((Graph *)self);
	}
	void set_title(const char *title) {
	  lg_graph_set_title((Graph *)self, title);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size((Graph *)self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y) {
	  lg_graph_set_size((Graph *)self, x, y);
	}
	int get_bg_color() {
	  lg_graph_get_bg_color((Graph *)self);
	}
	void set_bg_color(int color) {
	  lg_graph_set_bg_color((Graph *)self, color);
	}

	int add_pointname (char *name) {
	  lg_graph_add_pointname((Graph *)self, name);
	}
	void save (char *name) {
	  lg_graph_save((Graph *)self, name);
	}
  }
} MultiBar;

// libgraphmultiline.h
typedef struct lg_graph_multiline {
  %addmethods {
	MultiLine(char *title, int width, int height, int line) {
	  lg_graph_multiline_new(title, width, height, line);
	}
	~MultiLine() {
	  lg_graph_multiline_destroy (self);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(add_point) VALUE call_func_add_point(VALUE name, VALUE value) {
	  int i, len;
	  int *pnt;

	  Check_Type(value, T_ARRAY);
	  len = RARRAY(value)->len;
	  pnt = ALLOCA_N(int, len);
	  for (i = 0; i < len; i++) {
		pnt[i] = NUM2INT(RARRAY(value)->ptr[i]);
	  }
	  return INT2FIX(lg_graph_multiline_add_point (self, STR2CSTR(name), pnt));
	}
	%name(add_point_dbl) VALUE call_func_add_point_dbl(VALUE name, VALUE value) {
	  int i, len;
	  double *pnt;

	  Check_Type(value, T_ARRAY);
	  len = RARRAY(value)->len;
	  pnt = ALLOCA_N(double, len);
	  for (i = 0; i < len; i++) {
		pnt[i] = NUM2INT(RARRAY(value)->ptr[i]);
	  }
	  return INT2FIX(lg_graph_multiline_add_point_dbl (self, STR2CSTR(name), pnt));
	}
%endif
	void render ();

	// Graph
	char* get_title() {
	  lg_graph_get_title((Graph *)self);
	}
	void set_title(const char *title) {
	  lg_graph_set_title((Graph *)self, title);
	}
%ifdef SWIGRUBY // Ruby specific code
	%name(get_size) VALUE call_func_get_size() {
	  VALUE ary = rb_ary_new2(2);
	  int x, y;

	  lg_graph_get_size((Graph *)self, &x, &y);
	  rb_ary_push(ary, INT2FIX(x));
	  rb_ary_push(ary, INT2FIX(y));
	  return ary;
	}
%endif
	void set_size(int x, int y) {
	  lg_graph_set_size((Graph *)self, x, y);
	}
	int get_bg_color() {
	  lg_graph_get_bg_color((Graph *)self);
	}
	void set_bg_color(int color) {
	  lg_graph_set_bg_color((Graph *)self, color);
	}

	int add_pointname (char *name) {
	  lg_graph_add_pointname((Graph *)self, name);
	}
	void save (char *name) {
	  lg_graph_save((Graph *)self, name);
	}
  }
} MultiLine;
